const express    = require('express');
const bodyParser = require('body-parser');

const exec       = require('child_process').exec;
const execSync   = require('child_process').execSync;
const fs         = require('fs');
const path       = require('path');
const fsReaddir = require('fs-readdir');
const JSZip = require('jszip');

const exphbs = require('express-handlebars');

//initializer
const initGitRepos = require('./initializers/gitRepos.js');
//load names of available download files
const downloadFiles = require('./downloadFiles.json');
const downloadNames = Object.keys(downloadFiles);

//load functions for frontend and backend build
const clientFolder = require('./clientFolder.js');
const backendFunctions = require('./backendFunctions.js');
const frontendFunctions = require('./frontendFunctions.js');
const zipFunctions = require('./zipFunctions.js');

const app = express();

app.use(bodyParser.urlencoded({
	extended: true
}));
app.use(bodyParser.json());

//set handlebars as view engine
app.engine('.hbs', exphbs({extname: '.hbs'}));
app.set('view engine', '.hbs');

let clientCounter = 0;

	//TODO add spinner for telling user to wait until initialization is done
	//initGitRepos.initialize(downloadFiles);
	//routes
	app.get('/', function(req, res){

		res.render('home', {downloadFiles: downloadNames});
	});


	app.post('/', function(req, res, next){
		const checkedFiles = Object.keys(req.body);

		if(typeof checkedFiles !== 'undefined' && checkedFiles.length > 0){
          clientCounter= clientCounter + 1;
    console.log('clientCounter after increment:', clientCounter);
			res.render('download', {checkedFiles: checkedFiles, downloadFiles: downloadNames});
		}
		else {
			res.render('home', {downloadFiles: downloadNames});
		}
	});


	app.get('/download', function(req, res, next){
		let clientName;
		const zip = new JSZip();
		zip.file("README.txt", "text for README");
		//holds promises for backend and/or frontend for synchronizing
		const promiseArray = []
		const checkedFiles = req.query.checkedFiles.split(',');


		if (clientCounter > 5){
			res.render('tooManyClients');
		}else{
			clientName = 'client_' + clientCounter;
			clientFolder.makeClientFolder(clientName);
			
		}
		


		backendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-backend';
		});
		frontendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-frontend';
		});
//The user is not allowed to check plugins without having checked frontend or backend core. 
//If he has not checked at least one of those, he will be redirected to the homepage.
if(!(backendChecked || frontendChecked)){
	res.render('home', {downloadFiles: downloadNames});
}

if (backendChecked){

	const fileName = backendChecked;

	function backendAll(){
		return backendFunctions.prepareClientFolder(clientName,fileName).then(function(){
			return backendFunctions.backendClean(clientName, fileName).then(function(){
				return backendFunctions.backendPull(clientName, fileName).then(function(){
					return backendFunctions.backendInstallAddons(checkedFiles,downloadFiles,clientName).then(function(){
						return backendFunctions.backendBuild(clientName, fileName)
					})
				})
			})
		}).catch(function(error){
			let response = 
			'There was an error during your backend build. ' +
			'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
			'and add the following information: ' + error;
			res.send(response);

		})

	}
	promiseArray.push(backendAll());

} 



if (frontendChecked){
	const fileName = frontendChecked;
	const downloadURL = downloadFiles[frontendChecked]; 

	function frontendAll() {
		return frontendFunctions.prepareClientFolder(clientName,fileName).then(function(){
			return frontendFunctions.frontendClean(clientName,fileName).then(function(){
				return frontendFunctions.frontendPull(clientName,fileName).then(function(){
					return frontendFunctions.frontendInstallAddons(clientName,fileName, checkedFiles, downloadFiles).then(function(){
						return frontendFunctions.frontendBuild(clientName,fileName)
					})
				})
			})
		}).catch(function(error){
			let response = 
			'There was an error during your frontend build. ' +
			'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
			'and add the following information: ' + error;
			res.send(response);

		})
	}
	promiseArray.push(frontendAll());

};

finalizeZip();

console.log('promiseArray:', promiseArray);
function finalizeZip(){
	//Wo clientCounter runterzählen?
	return zipFunctions.streamFilesToZip(promiseArray,zip, clientName).then(function(zip){
		return createZip(zip)
	}).catch(function(error) {let response = 
		'There was an error during your build. ' +
		'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
		'and add the following information: ' + error;
		res.send(response);
	})

}

function createZip(zip){
	return new Promise((resolve,reject) => {
		zip.generateNodeStream({type:'nodebuffer',streamFiles:true}).pipe(fs.createWriteStream('explorviz-builds.zip')).on('error', function(error){
			console.log('error in createZip:', error);
		}).on('finish', function () {
			console.log('explorviz-builds.zip written.');
			console.log('clientCounter in createZip before decrement: ', clientCounter);
			clientCounter = clientCounter - 1;
			console.log('clientCounter in the end:', clientCounter);
			res.download('explorviz-builds.zip', function(error){
				if(error){
					let response = 
					'There was an error during your build. ' +
					'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
					'and add the following information: ' + error;
					res.send(response);
				} 
			});
		}); 

	})

	
}


});



	app.listen(3000);
	console.log("Downloader started on port 3000.");
